@@ -1,29 +1,33 @@
-# -*- coding: utf-8 -*-
 """
 Flow based cut algorithms
 """
+
 import itertools
+
 import networkx as nx
 
 # Define the default maximum flow function to use in all flow based
 # cut algorithms.
-from networkx.algorithms.flow import edmonds_karp
-from networkx.algorithms.flow import build_residual_network
+from networkx.algorithms.flow import build_residual_network, edmonds_karp
+
 default_flow_func = edmonds_karp
 
-from .utils import (build_auxiliary_node_connectivity,
-                    build_auxiliary_edge_connectivity)
+from .utils import build_auxiliary_edge_connectivity, build_auxiliary_node_connectivity
 
-__author__ = '\n'.join(['Jordi Torrents <jtorrents@milnou.net>'])
-
-__all__ = ['minimum_st_node_cut',
-           'minimum_node_cut',
-           'minimum_st_edge_cut',
-           'minimum_edge_cut']
+__all__ = [
+    "minimum_st_node_cut",
+    "minimum_node_cut",
+    "minimum_st_edge_cut",
+    "minimum_edge_cut",
+]
 
 
-def minimum_st_edge_cut(G, s, t, flow_func=None, auxiliary=None,
-                        residual=None):
+@nx._dispatchable(
+    graphs={"G": 0, "auxiliary?": 4},
+    preserve_edge_attrs={"auxiliary": {"capacity": float("inf")}},
+    preserve_graph_attrs={"auxiliary"},
+)
+def minimum_st_edge_cut(G, s, t, flow_func=None, auxiliary=None, residual=None):
     """Returns the edges of the cut-set of a minimum (s, t)-cut.
 
     This function returns the set of edges of minimum cardinality that,
@@ -49,10 +53,10 @@ def minimum_st_edge_cut(G, s, t, flow_func=None, auxiliary=None,
 
     flow_func : function
         A function for computing the maximum flow among a pair of nodes.
-        The function has to accept at least three parameters: a Digraph, 
-        a source node, and a target node. And return a residual network 
-        that follows NetworkX conventions (see :meth:`maximum_flow` for 
-        details). If flow_func is None, the default maximum flow function 
+        The function has to accept at least three parameters: a Digraph,
+        a source node, and a target node. And return a residual network
+        that follows NetworkX conventions (see :meth:`maximum_flow` for
+        details). If flow_func is None, the default maximum flow function
         (:meth:`edmonds_karp`) is used. See :meth:`node_connectivity` for
         details. The choice of the default function may change from version
         to version and should not be relied on. Default value: None.
@@ -95,25 +99,24 @@ def minimum_st_edge_cut(G, s, t, flow_func=None, auxiliary=None,
 
     If you need to compute local edge cuts on several pairs of
     nodes in the same graph, it is recommended that you reuse the
-    data structures that NetworkX uses in the computation: the 
+    data structures that NetworkX uses in the computation: the
     auxiliary digraph for edge connectivity, and the residual
     network for the underlying maximum flow computation.
 
     Example of how to compute local edge cuts among all pairs of
-    nodes of the platonic icosahedral graph reusing the data 
+    nodes of the platonic icosahedral graph reusing the data
     structures.
 
     >>> import itertools
-    >>> # You also have to explicitly import the function for 
+    >>> # You also have to explicitly import the function for
     >>> # building the auxiliary digraph from the connectivity package
-    >>> from networkx.algorithms.connectivity import (
-    ...     build_auxiliary_edge_connectivity)
+    >>> from networkx.algorithms.connectivity import build_auxiliary_edge_connectivity
     >>> H = build_auxiliary_edge_connectivity(G)
     >>> # And the function for building the residual network from the
     >>> # flow package
     >>> from networkx.algorithms.flow import build_residual_network
     >>> # Note that the auxiliary digraph has an edge attribute named capacity
-    >>> R = build_residual_network(H, 'capacity')
+    >>> R = build_residual_network(H, "capacity")
     >>> result = dict.fromkeys(G, dict())
     >>> # Reuse the auxiliary digraph and the residual network by passing them
     >>> # as parameters
@@ -143,7 +146,7 @@ def minimum_st_edge_cut(G, s, t, flow_func=None, auxiliary=None,
     else:
         H = auxiliary
 
-    kwargs = dict(capacity='capacity', flow_func=flow_func, residual=residual)
+    kwargs = {"capacity": "capacity", "flow_func": flow_func, "residual": residual}
 
     cut_value, partition = nx.minimum_cut(H, s, t, **kwargs)
     reachable, non_reachable = partition
@@ -156,6 +159,11 @@ def minimum_st_edge_cut(G, s, t, flow_func=None, auxiliary=None,
     return cutset
 
 
+@nx._dispatchable(
+    graphs={"G": 0, "auxiliary?": 4},
+    preserve_node_attrs={"auxiliary": {"id": None}},
+    preserve_graph_attrs={"auxiliary"},
+)
 def minimum_st_node_cut(G, s, t, flow_func=None, auxiliary=None, residual=None):
     r"""Returns a set of nodes of minimum cardinality that disconnect source
     from target in G.
@@ -175,12 +183,12 @@ def minimum_st_node_cut(G, s, t, flow_func=None, auxiliary=None, residual=None):
 
     flow_func : function
         A function for computing the maximum flow among a pair of nodes.
-        The function has to accept at least three parameters: a Digraph, 
-        a source node, and a target node. And return a residual network 
-        that follows NetworkX conventions (see :meth:`maximum_flow` for 
-        details). If flow_func is None, the default maximum flow function 
+        The function has to accept at least three parameters: a Digraph,
+        a source node, and a target node. And return a residual network
+        that follows NetworkX conventions (see :meth:`maximum_flow` for
+        details). If flow_func is None, the default maximum flow function
         (:meth:`edmonds_karp`) is used. See below for details. The choice
-        of the default function may change from version to version and 
+        of the default function may change from version to version and
         should not be relied on. Default value: None.
 
     auxiliary : NetworkX DiGraph
@@ -222,16 +230,15 @@ def minimum_st_node_cut(G, s, t, flow_func=None, auxiliary=None, residual=None):
     Example of how to compute local st node cuts reusing the data
     structures:
 
-    >>> # You also have to explicitly import the function for 
+    >>> # You also have to explicitly import the function for
     >>> # building the auxiliary digraph from the connectivity package
-    >>> from networkx.algorithms.connectivity import (
-    ...     build_auxiliary_node_connectivity)
+    >>> from networkx.algorithms.connectivity import build_auxiliary_node_connectivity
     >>> H = build_auxiliary_node_connectivity(G)
     >>> # And the function for building the residual network from the
     >>> # flow package
     >>> from networkx.algorithms.flow import build_residual_network
     >>> # Note that the auxiliary digraph has an edge attribute named capacity
-    >>> R = build_residual_network(H, 'capacity')
+    >>> R = build_residual_network(H, "capacity")
     >>> # Reuse the auxiliary digraph and the residual network by passing them
     >>> # as parameters
     >>> len(minimum_st_node_cut(G, 0, 6, auxiliary=H, residual=R))
@@ -254,7 +261,7 @@ def minimum_st_node_cut(G, s, t, flow_func=None, auxiliary=None, residual=None):
     is based in solving a number of maximum flow computations to determine
     the capacity of the minimum cut on an auxiliary directed network that
     corresponds to the minimum node cut of G. It handles both directed
-    and undirected graphs. This implementation is based on algorithm 11 
+    and undirected graphs. This implementation is based on algorithm 11
     in [1]_.
 
     See also
@@ -280,22 +287,22 @@ def minimum_st_node_cut(G, s, t, flow_func=None, auxiliary=None, residual=None):
     else:
         H = auxiliary
 
-    mapping = H.graph.get('mapping', None)
+    mapping = H.graph.get("mapping", None)
     if mapping is None:
-        raise nx.NetworkXError('Invalid auxiliary digraph.')
+        raise nx.NetworkXError("Invalid auxiliary digraph.")
     if G.has_edge(s, t) or G.has_edge(t, s):
-        return []
-    kwargs = dict(flow_func=flow_func, residual=residual, auxiliary=H)
+        return {}
+    kwargs = {"flow_func": flow_func, "residual": residual, "auxiliary": H}
 
     # The edge cut in the auxiliary digraph corresponds to the node cut in the
     # original graph.
-    edge_cut = minimum_st_edge_cut(H, '%sB' % mapping[s], '%sA' % mapping[t],
-                                   **kwargs)
+    edge_cut = minimum_st_edge_cut(H, f"{mapping[s]}B", f"{mapping[t]}A", **kwargs)
     # Each node in the original graph maps to two nodes of the auxiliary graph
-    node_cut = set(H.nodes[node]['id'] for edge in edge_cut for node in edge)
-    return node_cut - set([s, t])
+    node_cut = {H.nodes[node]["id"] for edge in edge_cut for node in edge}
+    return node_cut - {s, t}
 
 
+@nx._dispatchable
 def minimum_node_cut(G, s=None, t=None, flow_func=None):
     r"""Returns a set of nodes of minimum cardinality that disconnects G.
 
@@ -316,10 +323,10 @@ def minimum_node_cut(G, s=None, t=None, flow_func=None):
 
     flow_func : function
         A function for computing the maximum flow among a pair of nodes.
-        The function has to accept at least three parameters: a Digraph, 
-        a source node, and a target node. And return a residual network 
-        that follows NetworkX conventions (see :meth:`maximum_flow` for 
-        details). If flow_func is None, the default maximum flow function 
+        The function has to accept at least three parameters: a Digraph,
+        a source node, and a target node. And return a residual network
+        that follows NetworkX conventions (see :meth:`maximum_flow` for
+        details). If flow_func is None, the default maximum flow function
         (:meth:`edmonds_karp`) is used. See below for details. The
         choice of the default function may change from version
         to version and should not be relied on. Default value: None.
@@ -358,7 +365,7 @@ def minimum_node_cut(G, s=None, t=None, flow_func=None):
 
     If you need to perform several local st cuts among different
     pairs of nodes on the same graph, it is recommended that you reuse
-    the data structures used in the maximum flow computations. See 
+    the data structures used in the maximum flow computations. See
     :meth:`minimum_st_node_cut` for details.
 
     Notes
@@ -367,7 +374,7 @@ def minimum_node_cut(G, s=None, t=None, flow_func=None):
     is based in solving a number of maximum flow computations to determine
     the capacity of the minimum cut on an auxiliary directed network that
     corresponds to the minimum node cut of G. It handles both directed
-    and undirected graphs. This implementation is based on algorithm 11 
+    and undirected graphs. This implementation is based on algorithm 11
     in [1]_.
 
     See also
@@ -390,43 +397,43 @@ def minimum_node_cut(G, s=None, t=None, flow_func=None):
 
     """
     if (s is not None and t is None) or (s is None and t is not None):
-        raise nx.NetworkXError('Both source and target must be specified.')
+        raise nx.NetworkXError("Both source and target must be specified.")
 
     # Local minimum node cut.
     if s is not None and t is not None:
         if s not in G:
-            raise nx.NetworkXError('node %s not in graph' % s)
+            raise nx.NetworkXError(f"node {s} not in graph")
         if t not in G:
-            raise nx.NetworkXError('node %s not in graph' % t)
+            raise nx.NetworkXError(f"node {t} not in graph")
         return minimum_st_node_cut(G, s, t, flow_func=flow_func)
 
     # Global minimum node cut.
     # Analog to the algorithm 11 for global node connectivity in [1].
     if G.is_directed():
         if not nx.is_weakly_connected(G):
-            raise nx.NetworkXError('Input graph is not connected')
+            raise nx.NetworkXError("Input graph is not connected")
         iter_func = itertools.permutations
 
         def neighbors(v):
-            return itertools.chain.from_iterable([G.predecessors(v),
-                                                  G.successors(v)])
+            return itertools.chain.from_iterable([G.predecessors(v), G.successors(v)])
+
     else:
         if not nx.is_connected(G):
-            raise nx.NetworkXError('Input graph is not connected')
+            raise nx.NetworkXError("Input graph is not connected")
         iter_func = itertools.combinations
         neighbors = G.neighbors
 
     # Reuse the auxiliary digraph and the residual network.
     H = build_auxiliary_node_connectivity(G)
-    R = build_residual_network(H, 'capacity')
-    kwargs = dict(flow_func=flow_func, auxiliary=H, residual=R)
+    R = build_residual_network(H, "capacity")
+    kwargs = {"flow_func": flow_func, "auxiliary": H, "residual": R}
 
     # Choose a node with minimum degree.
     v = min(G, key=G.degree)
     # Initial node cutset is all neighbors of the node with minimum degree.
     min_cut = set(G[v])
     # Compute st node cuts between v and all its non-neighbors nodes in G.
-    for w in set(G) - set(neighbors(v)) - set([v]):
+    for w in set(G) - set(neighbors(v)) - {v}:
         this_cut = minimum_st_node_cut(G, v, w, **kwargs)
         if len(min_cut) >= len(this_cut):
             min_cut = this_cut
@@ -441,6 +448,7 @@ def minimum_node_cut(G, s=None, t=None, flow_func=None):
     return min_cut
 
 
+@nx._dispatchable
 def minimum_edge_cut(G, s=None, t=None, flow_func=None):
     r"""Returns a set of edges of minimum cardinality that disconnects G.
 
@@ -461,10 +469,10 @@ def minimum_edge_cut(G, s=None, t=None, flow_func=None):
 
     flow_func : function
         A function for computing the maximum flow among a pair of nodes.
-        The function has to accept at least three parameters: a Digraph, 
-        a source node, and a target node. And return a residual network 
-        that follows NetworkX conventions (see :meth:`maximum_flow` for 
-        details). If flow_func is None, the default maximum flow function 
+        The function has to accept at least three parameters: a Digraph,
+        a source node, and a target node. And return a residual network
+        that follows NetworkX conventions (see :meth:`maximum_flow` for
+        details). If flow_func is None, the default maximum flow function
         (:meth:`edmonds_karp`) is used. See below for details. The
         choice of the default function may change from version
         to version and should not be relied on. Default value: None.
@@ -483,10 +491,10 @@ def minimum_edge_cut(G, s=None, t=None, flow_func=None):
     >>> len(nx.minimum_edge_cut(G))
     5
 
-    You can use alternative flow algorithms for the underlying 
-    maximum flow computation. In dense networks the algorithm 
-    :meth:`shortest_augmenting_path` will usually perform better 
-    than the default :meth:`edmonds_karp`, which is faster for 
+    You can use alternative flow algorithms for the underlying
+    maximum flow computation. In dense networks the algorithm
+    :meth:`shortest_augmenting_path` will usually perform better
+    than the default :meth:`edmonds_karp`, which is faster for
     sparse networks with highly skewed degree distributions.
     Alternative flow functions have to be explicitly imported
     from the flow package.
@@ -503,7 +511,7 @@ def minimum_edge_cut(G, s=None, t=None, flow_func=None):
 
     If you need to perform several local computations among different
     pairs of nodes on the same graph, it is recommended that you reuse
-    the data structures used in the maximum flow computations. See 
+    the data structures used in the maximum flow computations. See
     :meth:`local_edge_connectivity` for details.
 
     Notes
@@ -512,7 +520,7 @@ def minimum_edge_cut(G, s=None, t=None, flow_func=None):
     undirected graphs the algorithm works by finding a 'small' dominating
     set of nodes of G (see algorithm 7 in [1]_) and computing the maximum
     flow between an arbitrary node in the dominating set and the rest of
-    nodes in it. This is an implementation of algorithm 6 in [1]_. For 
+    nodes in it. This is an implementation of algorithm 6 in [1]_. For
     directed graphs, the algorithm does n calls to the max flow function.
     The function raises an error if the directed graph is not weakly
     connected and returns an empty set if it is weakly connected.
@@ -537,19 +545,19 @@ def minimum_edge_cut(G, s=None, t=None, flow_func=None):
 
     """
     if (s is not None and t is None) or (s is None and t is not None):
-        raise nx.NetworkXError('Both source and target must be specified.')
+        raise nx.NetworkXError("Both source and target must be specified.")
 
     # reuse auxiliary digraph and residual network
     H = build_auxiliary_edge_connectivity(G)
-    R = build_residual_network(H, 'capacity')
-    kwargs = dict(flow_func=flow_func, residual=R, auxiliary=H)
+    R = build_residual_network(H, "capacity")
+    kwargs = {"flow_func": flow_func, "residual": R, "auxiliary": H}
 
     # Local minimum edge cut if s and t are not None
     if s is not None and t is not None:
         if s not in G:
-            raise nx.NetworkXError('node %s not in graph' % s)
+            raise nx.NetworkXError(f"node {s} not in graph")
         if t not in G:
-            raise nx.NetworkXError('node %s not in graph' % t)
+            raise nx.NetworkXError(f"node {t} not in graph")
         return minimum_st_edge_cut(H, s, t, **kwargs)
 
     # Global minimum edge cut
@@ -557,7 +565,7 @@ def minimum_edge_cut(G, s=None, t=None, flow_func=None):
     if G.is_directed():
         # Based on algorithm 8 in [1]
         if not nx.is_weakly_connected(G):
-            raise nx.NetworkXError('Input graph is not connected')
+            raise nx.NetworkXError("Input graph is not connected")
 
         # Initial cutset is all edges of a node with minimum degree
         node = min(G, key=G.degree)
@@ -579,7 +587,7 @@ def minimum_edge_cut(G, s=None, t=None, flow_func=None):
     else:  # undirected
         # Based on algorithm 6 in [1]
         if not nx.is_connected(G):
-            raise nx.NetworkXError('Input graph is not connected')
+            raise nx.NetworkXError("Input graph is not connected")
 
         # Initial cutset is all edges of a node with minimum degree
         node = min(G, key=G.degree)
