@@ -17,7 +17,7 @@ interact with different languages and even different Python versions.
 Re-importing from gml is also a concern.
 
 Without specifying a `stringizer`/`destringizer`, the code is capable of
-writing `int`/`float`/`str`/`dict`/`list` data as required by the GML 
+writing `int`/`float`/`str`/`dict`/`list` data as required by the GML
 specification.  For writing other data types, and for reading data other
 than `str` you need to explicitly supply a `stringizer`/`destringizer`.
 
@@ -27,6 +27,7 @@ For additional documentation on the GML file format, please see the
 Several example graphs in GML format may be found on Mark Newman's
 `Network data page <http://www-personal.umich.edu/~mejn/netdata/>`_.
 """
+
 import html.entities as htmlentitydefs
 import re
 import warnings
@@ -112,7 +113,7 @@ def literal_destringizer(rep):
 
 
 @open_file(0, mode="rb")
-@nx._dispatch(graphs=None)
+@nx._dispatchable(graphs=None, returns_graph=True)
 def read_gml(path, label="label", destringizer=None):
     """Read graph in GML format from `path`.
 
@@ -195,7 +196,7 @@ def read_gml(path, label="label", destringizer=None):
     return G
 
 
-@nx._dispatch(graphs=None)
+@nx._dispatchable(graphs=None, returns_graph=True)
 def parse_gml(lines, label="label", destringizer=None):
     """Parse GML graph from a string or iterable.
 
@@ -547,7 +548,7 @@ def literal_stringizer(value):
     """
 
     def stringize(value):
-        if isinstance(value, (int, bool)) or value is None:
+        if isinstance(value, int | bool) or value is None:
             if value is True:  # GML uses 1/0 for boolean values.
                 buf.write(str(1))
             elif value is False:
@@ -562,7 +563,7 @@ def literal_stringizer(value):
                 except UnicodeEncodeError:
                     text = "u" + text
             buf.write(text)
-        elif isinstance(value, (float, complex, str, bytes)):
+        elif isinstance(value, float | complex | str | bytes):
             buf.write(repr(value))
         elif isinstance(value, list):
             buf.write("[")
@@ -715,7 +716,7 @@ def generate_gml(G, stringizer=None):
         if not isinstance(key, str):
             key = str(key)
         if key not in ignored_keys:
-            if isinstance(value, (int, bool)):
+            if isinstance(value, int | bool):
                 if key == "label":
                     yield indent + key + ' "' + str(value) + '"'
                 elif value is True:
@@ -753,7 +754,7 @@ def generate_gml(G, stringizer=None):
                 yield indent + "]"
             elif isinstance(value, tuple) and key == "label":
                 yield indent + key + f" \"({','.join(repr(v) for v in value)})\""
-            elif isinstance(value, (list, tuple)) and key != "label" and not in_list:
+            elif isinstance(value, list | tuple) and key != "label" and not in_list:
                 if len(value) == 0:
                     yield indent + key + " " + f'"{value!r}"'
                 if len(value) == 1:
